/**********************************************************************
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
************************************************************************
	(c) 2008 by Mister P and Q  < Mister.PandQ@gmail.com >
***********************************************************************/

#define _GNU_SOURCE
#include <stdio.h>
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <openssl/bn.h>
#include <openssl/crypto.h>

void print_prime(char *p_acLabel, char *prime)
{
	int i;
	for(i = 0; i < strlen(prime) && prime[i] == '0'; i++);
	printf("\n%s",p_acLabel);
	for(; i < strlen(prime); i++)
		printf("%c", prime[i]);
	printf("\n");
}

char *acDidierKey = "D0EA1ABA978DF0065B2009F75C846F28B04ED5143B237B3FC24272245ADE837EFE0271E1A2854E0C81BA9F70A83AD86D47B0EACD062BC15BC61A99DC83124EC9";


int main(int argc, char *argv[])
{
	char	*p_acPrime = NULL;
	time_t	tStart, tNow;		
	float	fPerformance;
	BIGNUM *p_tBNtmp, *p_tBNDidier, *p_tBNRemain, *p_tBNResult = NULL;
	BN_CTX *p_tBNCtx = NULL;
	
	// We assume p & Q are 256bits long
	long int lNumBits = 256L;
	long int lCount	= 0L;	
	
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_IDLE);

	// Allocate and Init context
	p_tBNCtx = BN_CTX_new();
	BN_CTX_init(p_tBNCtx);

	// Allocate BigNum structures
	p_tBNDidier = BN_new();
	p_tBNRemain	= BN_new();
	p_tBNResult = BN_new();
	p_tBNtmp	= BN_new();

	// Convert Didier's Key to a bignum structure
	BN_hex2bn(&p_tBNDidier, acDidierKey);

	// Print Didier's Key in Dec
	p_acPrime = BN_bn2dec(p_tBNDidier);
	print_prime("Didier's Key [DEC]:",p_acPrime);
	OPENSSL_free(p_acPrime);

	// Get starting time
	time(&tStart);

	// Loop until P & Q found
	for (;;) 
		{
		// Generate a 512bit random prime number
		BN_generate_prime(p_tBNtmp,lNumBits,1,NULL,NULL,NULL,NULL);
		lCount++;

		// Divide Didier's key by random generated prime
		BN_div(p_tBNResult, p_tBNRemain, p_tBNDidier, p_tBNtmp, p_tBNCtx);

		// Check if the remainder is zero
		// if so, we have found one prime that makes up
		// the modulus
		if (BN_is_zero(p_tBNRemain))
			{
			printf("Remainder is zero\n");
			p_acPrime = BN_bn2dec(p_tBNResult);
			print_prime("P [DEC]: ", p_acPrime);
			OPENSSL_free(p_acPrime);

			p_acPrime = BN_bn2dec(p_tBNtmp);
			print_prime("Q [DEC]: ",p_acPrime);
			OPENSSL_free(p_acPrime);
			break;
			}
		if ((lCount % 100) == 0)
			{
			time(&tNow);
			fPerformance = lCount;
			fPerformance/= (tNow - tStart);
			printf("Performance = [%.2f] Keys/second\n",fPerformance);
			}
	}
	BN_free(p_tBNtmp);
	BN_free(p_tBNResult);
	BN_free(p_tBNRemain);
	BN_free(p_tBNDidier);
	
	BN_CTX_free(p_tBNCtx);
}
